package ru.nsu.fit.mustakov.graphics;

import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.util.HashSet;
import java.util.Set;

public abstract class Plot {
    private Set<PlotChangeListener> listeners;
    
    protected Plot() {
        listeners = new HashSet<PlotChangeListener>();
    }

    protected final void dispatchChangeEvent() {
        PlotChangeEvent changeEvent = new PlotChangeEvent(this);
        for (PlotChangeListener listener : listeners) {
            listener.plotChanged(changeEvent);
        }
    }

    public final void addChangeListener(PlotChangeListener changeListener) {
        listeners.add(changeListener);
    }

    public final void removeChangeListener(PlotChangeListener changeListener) {
        listeners.remove(changeListener);
    }

    public abstract void draw(Graphics2D g2, Rectangle2D area);

    public void zoom(double percent) {

    }
}
