package ru.nsu.fit.mustakov.graphics;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.Rectangle2D;

public class PlotMaker extends JFrame {
    PlotPanel plotPanel;
    
    public PlotMaker(){
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        initContentPane();
        pack();
        setLocationRelativeTo(null);
        setVisible(true);
    }
    
    private void initContentPane(){
        plotPanel = new PlotPanel();
        plotPanel.setFunction(new LogarithmFunction());
        getContentPane().add(plotPanel);
    }

    class PlotPanel extends JPanel implements MouseWheelListener{

        private static final int PREFERRED_WIDTH = 800;
        private static final int PREFERRED_HEIGHT = 800;

        private final static double SCROLL_SENSITIVITY = 0.05;
        
        private Rectangle2D area;
        private FunctionPlot plot;
        
        public PlotPanel()
        {
            this(PREFERRED_WIDTH, PREFERRED_HEIGHT);
        }
        
        public PlotPanel(int width, int height) {
            super();
            plot = new FunctionPlot();
            area = new Rectangle(0, 0, width, height);
            addMouseWheelListener(this);
        } 
        
        @Override
        public void paintComponent(Graphics g)
        {
            super.paintComponent(g);
            setBackground(Color.white);
            Graphics2D g2 = (Graphics2D) g;
            plot.draw(g2, area);
        }
        
        @Override
        public Dimension getPreferredSize(){
            return new Dimension((int)area.getWidth(), (int)area.getHeight());
        }
        
        public void setFunction(MathFunction function) {
            plot.setFunction(function);
        }

        @Override
        public void mouseWheelMoved(MouseWheelEvent e) {
            if (e.getScrollType() == MouseWheelEvent.WHEEL_UNIT_SCROLL) {
                int scrollAmount = e.getUnitsToScroll();
                int notches = e.getWheelRotation();
                if (notches < 0) {
                    plot.zoom(plot.getZoom() - scrollAmount * SCROLL_SENSITIVITY);
                }
                else {
                    double scrollSensitivity = SCROLL_SENSITIVITY;
                    while (plot.getZoom() <= scrollAmount * scrollSensitivity) {
                        scrollSensitivity /= 2;
                    }
                    plot.zoom(plot.getZoom() - scrollAmount * scrollSensitivity);
                }
                repaint();
            }
        }
    }
}
